define bi_ndenv::bower::install (
    $package_name = $title,
    $version = undef,
    $save = undef,
    $save_dev = undef,
    $save_exact = undef,
    $production = undef,
    $force_latest = undef,
    $cwd = undef,
    $user = undef,
    $http_proxy = undef,
    $https_proxy = undef,
    $node_version = undef,
    $bower_version = '1.4.1',
) {
    require ::bi_ndenv
    require ::bi_grep
    require ::bi_bash
    
    if $node_version == undef or !is_string($node_version) { 
        fail("'node_version' must be defined") 
    }
    
    if $cwd == undef or !is_string($cwd) { 
        fail("'cwd' must be defined") 
    }

    if $package_name != undef and !is_string($package_name) { 
        fail("'package_name' must be a string") 
    }
    
    if $user == undef or !is_string($user) { 
        fail("'user' must be defined") 
    }
    
    if $version != undef and !is_string($version) { 
        fail("'version' must be a string [ $version ]") 
    }
    
    if !$package_name and $version {
        fail("'version' can't be defined if 'package' is undefined")
    }
    
    if $save != undef and !is_bool($save) { 
        fail("'save' must be a bool") 
    }
    
    if $save_dev != undef and !is_bool($save_dev) { 
        fail("'save_dev' must be a bool") 
    }
    
    if $production != undef and !is_bool($production) { 
        fail("'production' must be a bool") 
    }
    
    if $save_exact != undef and !is_bool($save_exact) { 
        fail("'save_exact' must be a bool") 
    }
    
    if $force_latest != undef and !is_bool($force_latest) { 
        fail("'force_latest' must be a bool") 
    }
    
    $save_args = delete_undef_values([ 
        $save, 
        $save_dev,
        $save_exact,
    ])
    
    if size($save_args) > 1 {
        fail("mutually exclusive arguments: 'save', 'save_dev', 'save_exact'")
    }
    
    $arg_save_exact = $save_exact ? {
        true => '--save-exact',
        default => undef,
    }
    
    $arg_production = $production ? {
        true => '--production',
        default => undef,
    }
    
    $arg_force_latest = $force_latest ? {
        true => '--force-latest',
        default => undef,
    }

    $arg_save = $save ? {
        true => '--save',
        default => undef,
    }
    
    $arg_save_dev = $save_dev ? {
        true => '--save-dev',
        default => undef,
    }
    
    if $http_proxy != undef and is_string($http_proxy) {
        $arg_http_proxy = "--http-proxy=$http_proxy"
    } else {
        $arg_http_proxy = ""
    }
    
    if $https_proxy != undef and is_string($https_proxy) {
        $arg_https_proxy = "--https-proxy=$https_proxy"
    } else {
        $arg_https_proxy = ""
    }
    
    $args = join(delete_undef_values([ 
        $arg_save_exact, 
        $arg_production,
        $arg_save,
        $arg_save_dev,
        $arg_force_latest,
    ]), " ")
    
    if $package_name {
        if $version {
            $arg_version = "#$version"
        } else {
            $arg_version = ""
        }
        
        $fqpn = "${package_name}${arg_version}"
    } else {
        $fqpn = ""
    }
    
    $command = "bower $arg_http_proxy $arg_https_proxy install $fqpn $args"
    debug("command: $command")

    if $package_name {
        $unless = "bower list | $::bi_grep::exe_path '$fqpn'"   
    } else {
        $unless = undef
    }
    debug("unless: $unless")
    
     bi_ndenv::npm::package { "$user/bower":
        package_name => 'bower',
        version => $bower_version,
        user => $user,
        cwd => $cwd,
        ensure => 'present',
        global => true,
        http_proxy => $http_proxy,
        https_proxy => $https_proxy,
    }
    ->
    bi_bash::exec { "$user/$command":
        command => $command,
        unless => $unless,
        user => $user,
        cwd => $cwd,
    }
}
