define bi_ndenv::npm::uninstall (
    $package_name = $title,
    $scope = undef,
    $save = undef,
    $save_dev = undef,
    $save_optional = undef,
    $save_exact = undef,
    $cwd = undef,
    $user = undef,
    $global = false,
) {
    require bi_ndenv
    require bi_grep
    require bi_bash
    
    if $global == undef or !is_bool($global) { 
        fail("'global' must be defined") 
    }
    
    if $cwd == undef or !is_string($cwd) { 
        fail("'cwd' must be defined") 
    }
    
    if $package_name == undef or !is_string($package_name) { 
        fail("'package_name' must be defined") 
    }
    
    if $user == undef or !is_string($user) { 
        fail("'user' must be defined") 
    }
    
    if $scope != undef and !is_string($scope) { 
        fail("'scope' must be a string") 
    }
    
    if $save != undef and !is_bool($save) { 
        fail("'save' must be a bool") 
    }
    
    if $save_dev != undef and !is_bool($save_dev) { 
        fail("'save_dev' must be a bool") 
    }
    
    if $save_optional != undef and !is_bool($save_optional) { 
        fail("'save_optional' must be a bool") 
    }
    
    if $save_exact != undef and !is_bool($save_exact) { 
        fail("'save_exact' must be a bool") 
    }
    
    $save_args = delete_undef_values([ 
        $save, 
        $save_dev,
        $save_optional,
        $save_exact,
    ])
    
    if size($save_args) > 1 {
        fail("mutually exclusive arguments: 'save', 'save_dev', 'save_optional', 'save_exact'")
    }
    
    $arg_save_exact = $save_exact ? {
        true => '--save-exact',
        default => undef,
    }
    
    $arg_save_optional = $save_optional ? {
        true => '--save-optional',
        default => undef,
    }

    $arg_save = $save ? {
        true => '--save',
        default => undef,
    }
    
    $arg_save_dev = $save_dev ? {
        true => '--save-dev',
        default => undef,
    }
    
    $arg_global = $global ? {
        true => '--global',
        default => undef,
    }

    $args = join(delete_undef_values([ 
        $arg_save_exact, 
        $arg_save_optional,
        $arg_save,
        $arg_save_dev,
        $arg_global,
    ]), " ")
    
    if $scope {
        $arg_scope = "@$scope/"
    } else {
        $arg_scope = ""
    }
    
    $fqpn = "${arg_scope}${package_name}"
    $command = "npm uninstall $fqpn $args"
    debug("command: $command")
    
    if $arg_global {
        $unless = "npm list $arg_global | $::bi_grep::exe_path -v '$fqpn'"
    } else {
        $unless = "npm list | $::bi_grep::exe_path -v '$fqpn'"
    }
    debug("unless: $unless")
    
    bi_bash::exec { $command:
        unless => $unless,
        user => $user,
        cwd => $cwd,
    }
}
